@@ -35,6 +35,13 @@ static GtkTargetEntry drop_types[] = {
   /*  { "x-special/gnome-reset-background", 0, TARGET_BACKGROUND_RESET }*/
 };
 
+static gchar **command_line_files = NULL;
+
+static const GOptionEntry options[] = {
+  { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &command_line_files, NULL,  N_("[FILE...]") },
+  { NULL }
+};
+
 static void wp_props_load_wallpaper (gchar * key,
 				     GnomeWPItem * item,
 				     GnomeWPCapplet * capplet);
@@ -278,7 +285,6 @@ static gboolean gnome_wp_props_wp_set (GnomeWPCapplet * capplet) {
   GnomeWPItem * item;
   GConfChangeSet * cs;
   gchar * wpfile;
-  gboolean retval = FALSE;
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (capplet->treeview));
   if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
@@ -286,79 +292,38 @@ static gboolean gnome_wp_props_wp_set (GnomeWPCapplet * capplet) {
 
     item = g_hash_table_lookup (capplet->wphash, wpfile);
 
-    if (g_utf8_collate (capplet->old_item->filename, wpfile) != 0) {
-      cs = gconf_change_set_new ();
+    cs = gconf_change_set_new ();
 
-      if (!strcmp (item->filename, "(none)")) {
-	gconf_change_set_set_string (cs, WP_OPTIONS_KEY, "none");
-	gconf_change_set_set_string (cs, WP_FILE_KEY, "");
-      } else {
-	gchar * uri;
+    if (!strcmp (item->filename, "(none)")) {
+      gconf_change_set_set_string (cs, WP_OPTIONS_KEY, "none");
+      gconf_change_set_set_string (cs, WP_FILE_KEY, "");
+    } else {
+      gchar * uri;
 
-	if (g_utf8_validate (item->filename, -1, NULL))
-	  uri = g_strdup (item->filename);
-	else
-	  uri = g_filename_to_utf8 (item->filename, -1, NULL, NULL, NULL);
+      if (g_utf8_validate (item->filename, -1, NULL))
+	uri = g_strdup (item->filename);
+      else
+	uri = g_filename_to_utf8 (item->filename, -1, NULL, NULL, NULL);
 
-	gconf_change_set_set_string (cs, WP_FILE_KEY, uri);
-	g_free (uri);
+      gconf_change_set_set_string (cs, WP_FILE_KEY, uri);
+      g_free (uri);
 
-	gconf_change_set_set_string (cs, WP_OPTIONS_KEY, item->options);
-      }
-
-      gconf_change_set_set_string (cs, WP_SHADING_KEY, item->shade_type);
-
-      gconf_change_set_set_string (cs, WP_PCOLOR_KEY, item->pri_color);
-      gconf_change_set_set_string (cs, WP_SCOLOR_KEY, item->sec_color);
-
-      gconf_client_commit_change_set (capplet->client, cs, TRUE, NULL);
-
-      gconf_change_set_unref (cs);
-
-      retval = TRUE;
+      gconf_change_set_set_string (cs, WP_OPTIONS_KEY, item->options);
     }
+
+    gconf_change_set_set_string (cs, WP_SHADING_KEY, item->shade_type);
+
+    gconf_change_set_set_string (cs, WP_PCOLOR_KEY, item->pri_color);
+    gconf_change_set_set_string (cs, WP_SCOLOR_KEY, item->sec_color);
+
+    gconf_client_commit_change_set (capplet->client, cs, TRUE, NULL);
+
+    gconf_change_set_unref (cs);
+
     g_free (wpfile);
   }
-  return retval;
-}
 
-static gboolean gnome_wp_props_revert (GnomeWPCapplet * capplet) {
-  GnomeWPItem * item;
-  GConfChangeSet * cs;
-  gboolean retval = FALSE;
-
-  item = capplet->old_item;
-
-  cs = gconf_change_set_new ();
-
-  if (!strcmp (item->filename, "(none)")) {
-    gconf_change_set_set_string (cs, WP_OPTIONS_KEY, "none");
-  } else {
-    gchar * uri;
-
-    if (g_utf8_validate (item->filename, -1, NULL))
-      uri = g_strdup (item->filename);
-    else
-      uri = g_filename_to_utf8 (item->filename, -1, NULL, NULL, NULL);
-
-    gconf_change_set_set_string (cs, WP_FILE_KEY, uri);
-    g_free (uri);
-
-    gconf_change_set_set_string (cs, WP_OPTIONS_KEY, item->options);
-  }
-
-  gconf_change_set_set_string (cs, WP_SHADING_KEY, item->shade_type);
-
-  gconf_change_set_set_string (cs, WP_PCOLOR_KEY, item->pri_color);
-  gconf_change_set_set_string (cs, WP_SCOLOR_KEY, item->sec_color);
-
-  gconf_client_commit_change_set (capplet->client, cs, TRUE, NULL);
-
-  gconf_change_set_unref (cs);
-
-  retval = TRUE;
-
-  return retval;
+  return FALSE;
 }
 
 static void gnome_wp_props_wp_selected (GtkTreeSelection * selection,
@@ -389,6 +354,8 @@ static void gnome_wp_props_wp_selected (GtkTreeSelection * selection,
   } else {
     gtk_widget_set_sensitive (capplet->rm_button, FALSE);
   }
+
+  gnome_wp_props_wp_set (capplet);
 }
 
 static void gnome_wp_remove_wp (gchar * key, GnomeWPItem * item,
@@ -408,9 +375,7 @@ static void gnome_wp_remove_wp (gchar * key, GnomeWPItem * item,
 }
 
 void gnome_wp_main_quit (GnomeWPCapplet * capplet) {
-  gnome_wp_item_free (capplet->old_item);
-  g_hash_table_foreach (capplet->wphash, (GHFunc) gnome_wp_remove_wp,
-			capplet);
+  g_hash_table_foreach (capplet->wphash, (GHFunc) gnome_wp_remove_wp, capplet);
 
   gnome_wp_xml_save_list (capplet);
 
@@ -429,35 +394,12 @@ static void wallpaper_properties_clicked (GtkWidget * dialog,
     wp_properties_help (GTK_WINDOW (dialog),
 			"user-guide.xml", "goscustdesk-7");
     break;
-  case GTK_RESPONSE_APPLY:
-    gnome_wp_props_wp_set (capplet);
-    break;
-  case GTK_RESPONSE_OK:
-    if (gnome_wp_props_wp_set (capplet)) {
-      /*
-	 This is here because applying the background is about 1.4 seconds
-	 slow, with nautilus managing the desktop on my Radeon 7500
-	 Without nautilus, this is about 600000 usecs
-      */
-      usleep (1400000);
-    }
-    gtk_widget_destroy (dialog);
-    gnome_wp_main_quit (capplet);
-    break;
   case GTK_RESPONSE_DELETE_EVENT:
-  case GTK_RESPONSE_CANCEL: {
-    if (gnome_wp_props_revert (capplet)) {
-      /*
-	  Reverting the preferences will also have the same slowness as
-	  applying them and quitting, so we need a timeout here as well
-       */
-      usleep (1400000);
-    }
+  case GTK_RESPONSE_OK:
     gtk_widget_destroy (dialog);
     gnome_wp_main_quit (capplet);
     break;
   }
-  }
 }
 
 static void gnome_wp_scale_type_changed (GtkOptionMenu * option_menu,
@@ -505,6 +447,8 @@ static void gnome_wp_scale_type_changed (GtkOptionMenu * option_menu,
 		      0, pixbuf,
 		      -1);
   g_object_unref (pixbuf);
+  gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
+			   item->options, NULL);
 }
 
 static void gnome_wp_shade_type_changed (GtkOptionMenu * option_menu,
@@ -550,6 +494,8 @@ static void gnome_wp_shade_type_changed (GtkOptionMenu * option_menu,
 		      0, pixbuf,
 		      -1);
   g_object_unref (pixbuf);
+  gconf_client_set_string (capplet->client, WP_SHADING_KEY,
+			   item->shade_type, NULL);
 }
 
 static void gnome_wp_color_changed (GnomeWPCapplet * capplet,
@@ -591,6 +537,13 @@ static void gnome_wp_color_changed (GnomeWPCapplet * capplet,
 				     item->scolor->green >> 8,
 				     item->scolor->blue >> 8);
 
+  if (update) {
+    gconf_client_set_string (capplet->client, WP_PCOLOR_KEY,
+			     item->pri_color, NULL);
+    gconf_client_set_string (capplet->client, WP_SCOLOR_KEY,
+			     item->sec_color, NULL);
+  }
+
   gnome_wp_shade_type_changed (NULL, capplet);
 }
 
@@ -654,7 +607,6 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 
   item = g_hash_table_lookup (capplet->wphash, imagepath);
   if (item != NULL && strcmp (style, "none") != 0) {
-    capplet->old_item = gnome_wp_item_dup (item);
     if (item->deleted == TRUE) {
       item->deleted = FALSE;
       wp_props_load_wallpaper (item->filename, item, capplet);
@@ -687,10 +639,6 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 
     if (!strcmp (style, "none")) {
       gnome_wp_capplet_scroll_to_item (capplet, item);
-      if (capplet->old_item)
-	gnome_wp_item_free (capplet->old_item);
-
-      capplet->old_item = gnome_wp_item_dup (item);
     }
   }
   g_free (imagepath);
@@ -701,9 +649,6 @@ static gboolean gnome_wp_load_stuffs (void * data) {
     g_slist_free (capplet->uri_list);
   }
 
-  if (capplet->old_item == NULL && item != NULL)
-    capplet->old_item = gnome_wp_item_dup (item);
-
   return FALSE;
 }
 
@@ -848,23 +793,6 @@ static void gnome_wp_color2_changed (GConfClient * client, guint id,
   gnome_wp_color_changed (capplet, FALSE);
 }
 
-static void gnome_wp_icon_theme_changed (GtkIconTheme * theme,
-					 GnomeWPCapplet * capplet) {
-  GdkPixbuf * pixbuf;
-  GtkIconInfo * icon_info = NULL;
-
-  icon_info = gtk_icon_theme_lookup_icon (capplet->theme,
-					  "gnome-settings-background",
-					  48, 0);
-  if (icon_info != NULL) {
-    pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
-    gtk_icon_info_free (icon_info);
-    gtk_window_set_icon (GTK_WINDOW (capplet->window), NULL);
-    gtk_window_set_default_icon (pixbuf);
-    g_object_unref (pixbuf);
-  }
-}
-
 static GladeXML * gnome_wp_create_dialog (void) {
   GladeXML * new;
   gchar * gladefile;
@@ -892,13 +820,15 @@ static void gnome_wp_update_preview (GtkFileChooser *chooser,
   uri = gtk_file_chooser_get_preview_uri (chooser);
 
   if (uri) {
-    GdkPixbuf *pixbuf;
+    GdkPixbuf *pixbuf = NULL;
     gchar *mime_type;
 
     mime_type = gnome_vfs_get_mime_type (uri);
-    pixbuf = gnome_thumbnail_factory_generate_thumbnail (capplet->thumbs,
+    if (mime_type) {
+      pixbuf = gnome_thumbnail_factory_generate_thumbnail (capplet->thumbs,
 							 uri,
 							 mime_type);
+    }
 
     if(pixbuf != NULL) {
       gtk_image_set_from_pixbuf (GTK_IMAGE (capplet->image), pixbuf);
@@ -913,7 +843,7 @@ static void gnome_wp_update_preview (GtkFileChooser *chooser,
   gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
 }
   
-static void wallpaper_properties_init (poptContext ctx) {
+static void wallpaper_properties_init () {
   GnomeWPCapplet * capplet;
   GladeXML * dialog;
   GtkWidget * menu;
@@ -922,10 +852,7 @@ static void wallpaper_properties_init (poptContext ctx) {
   GtkCellRenderer * renderer;
   GtkTreeViewColumn * column;
   GtkTreeSelection * selection;
-  GdkPixbuf * pixbuf;
   GdkCursor * cursor;
-  const gchar ** args;
-  GtkIconInfo * icon_info = NULL;
   GtkFileFilter *filter;
 
   gtk_rc_parse_string ("style \"wp-tree-defaults\" {\n"
@@ -933,6 +860,7 @@ static void wallpaper_properties_init (poptContext ctx) {
 		       "} widget_class \"*TreeView*\""
 		       " style \"wp-tree-defaults\"\n"
 		       "style \"wp-dialog-defaults\" {\n"
+		       "  GtkDialog::content-area-border = 0\n"
 		       "  GtkDialog::action-area-border = 12\n"
 		       "} widget_class \"*Dialog*\""
 		       " style \"wp-dialog-defaults\"\n");
@@ -975,31 +903,14 @@ static void wallpaper_properties_init (poptContext ctx) {
 					   gnome_wp_item_free);
 
   capplet->thumbs = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
-  capplet->theme = gtk_icon_theme_get_default ();
-
-  g_signal_connect (G_OBJECT (capplet->theme), "changed",
-		    G_CALLBACK (gnome_wp_icon_theme_changed), capplet);
 
   dialog = gnome_wp_create_dialog ();
   capplet->window = glade_xml_get_widget (dialog, "gnome_wp_properties");
 
-  icon_info = gtk_icon_theme_lookup_icon (capplet->theme,
-					  "gnome-settings-background",
-					  48, 0);
-  if (icon_info != NULL) {
-    pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
-    gtk_icon_info_free (icon_info);
-    gtk_window_set_default_icon (pixbuf);
-    gtk_window_set_icon (GTK_WINDOW (capplet->window), pixbuf);
-    g_object_unref (pixbuf);
-  }
+  gtk_window_set_default_icon_name ("preferences-desktop-wallpaper");
 
   gtk_widget_realize (capplet->window);
 
-  gtk_widget_ensure_style (capplet->window);
-  gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (capplet->window)->vbox), 0);
-  gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (capplet->window)->action_area), 12);
-
   /* Drag and Drop Support */
   gtk_drag_dest_unset (capplet->window);
   gtk_drag_dest_set (capplet->window, GTK_DEST_DEFAULT_ALL, drop_types,
@@ -1114,17 +1025,14 @@ static void wallpaper_properties_init (poptContext ctx) {
   gdk_window_set_cursor (capplet->window->window, cursor);
   gdk_cursor_unref (cursor);
 
-  args = poptGetArgs (ctx);
-  if (args != NULL) {
+  if (command_line_files != NULL) {
     const gchar ** p;
 
-    for (p = args; *p != NULL; p++) {
+    for (p = (const gchar **) command_line_files; *p != NULL; p++) {
       capplet->uri_list = g_slist_append (capplet->uri_list, (gchar *) *p);
     }
   }
 
-  poptFreeContext (ctx);
-
   g_idle_add (gnome_wp_load_stuffs, capplet);
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (capplet->treeview));
@@ -1216,24 +1124,25 @@ static void gnome_wp_set_sensitivities (GnomeWPCapplet* capplet) {
 
 gint main (gint argc, gchar *argv[]) {
   GnomeProgram * program;
-  GValue context = { 0 };
-  poptContext ctx;
+  GOptionContext *ctx;
+
+  ctx = g_option_context_new (_("- Desktop Background Preferences"));
 
 #ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
+  g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
+#else
+  g_option_context_add_main_entries (ctx, options, NULL);
 #endif
 
-  program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
-				argc, argv, GNOME_PARAM_POPT_TABLE,
-				NULL, NULL);
+  program = gnome_program_init ("gnome-background-properties", VERSION, 
+		  		LIBGNOMEUI_MODULE, argc, argv,
+				GNOME_PARAM_GOPTION_CONTEXT, ctx,
+				GNOME_PARAM_NONE);
 
-  g_object_get_property (G_OBJECT (program), GNOME_PARAM_POPT_CONTEXT,
-			 g_value_init (&context, G_TYPE_POINTER));
-  ctx = g_value_get_pointer (&context);
-
-  wallpaper_properties_init (ctx);
+  wallpaper_properties_init ();
 
   gtk_main ();
   
